<?php

class CACHEReader extends READER {
 var $cache;

 function __construct(&$props, CACHEDB &$cache = NULL) {
    parent::__construct($props);
    if ($cache) $this->cache = &$cache;
    else {
	if ($props instanceof SOURCERequest)
	    $this->cache = new CACHEDB($props);
	else
	    $this->cache = NULL;
    }
 }
 
 function GetGroupInfo(LOGGROUP $grp = NULL, $flags = 0) {
    $groups = array();

    $res = $this->cache->ListCachedGroups();
    foreach ($res as $gid) {
	if (($grp)&&(strcmp($grp->gid, $gid))) continue;
	
	$groups[$gid] = array(
	    'gid' => $gid,
	    'name' => $gid
	);

	if ($flags&REQUEST::NEED_INFO) {
	    $postfix = $this->cache->GetCachePostfix($gid);
	    
	    $info = $this->cache->GetCacheInfo($postfix, $flags&REQUEST::FLAG_MASK);

	    if (!$info)
		throw new ADEIException(translate("The CACHE for group (%s) is empty", $grp->gid));

	    foreach ($info as $key => &$value) {
		$groups[$gid][$key] = $value;
	    }
	    
	    if ($flags&REQUEST::NEED_ITEMINFO) {
		$groups[$gid]['items'] = $this->cache->GetCacheItemList($postfix);
	    }
	}
    }

    return $grp?$groups[$grp->gid]:$groups;
 }
 
 function GetItemList(LOGGROUP $grp = NULL, MASK $mask = NULL, $flags = 0) {
    $grp = $this->CheckGroup($grp);

    $postfix = $this->cache->GetCachePostfix($grp->gid);
    if (!$mask) $mask = $this->CreateMask($grp);

    return $this->cache->GetCacheItemList($postfix, $mask, 0);
 }

 function GetRawData(LOGGROUP $grp = NULL, $from = 0, $to = 0, DATAFilter $filter = NULL, &$filter_data = NULL) {
    $grp = $this->CheckGroup($grp);

    $postfix = $this->cache->GetCachePostfix($grp->gid);

    $ivl = $this->CreateInterval($grp);
    $ivl->Limit($from, $to);

    if ($filter) {
	$limit = $filter->GetVectorsLimit();
	$resample = $filter->GetSamplingRate();
	$mask = $filter->GetItemMask();
	
	if (isset($filter_data)) {
	    if ($mask) $filter_data['masked'] = true;
	    if ($resample) $filter_data['resampled'] = true;
	    if ($limit) $filter_data['limited'] = true;
	}
    } else {
	$mask = NULL;
	$resample = 0;
	$limit = 0;
    }

    if (!$mask) {
	$minfo = array("db_mask"=>"");
	$mask = $this->cache->CreateCacheMask($postfix, $minfo);
    }

    return $this->cache->GetCachePoints($postfix, $mask, $ivl, CACHEDB::TYPE_ALL, $limit, $resample);
 }

 function HaveData(LOGGROUP $grp = NULL, $from = 0, $to = 0) {
    $grp = $this->CheckGroup($grp);

    $postfix = $this->cache->GetCachePostfix($grp->gid);

    $ivl = $this->CreateInterval($grp);
    $ivl->Limit($from, $to);

    $mask = NULL;

    $points = $this->cache->GetCachePoints($postfix, $mask, $ivl, CACHEDB::TYPE_ALL, 1);

    $points->rewind();
    return $points->valid();
 }
}

?>