function SOURCE_CONFIG() {
    this.db_server = null;
    this.db_name = null;
    this.db_group = null;
    this.db_mask = null;
    this.db_item = null;
}


function SOURCE(interval, srv_sel_id, db_sel_id, group_sel_id, mask_sel_id, item_sel_id, opts) {
    this.cfg = new SOURCE_CONFIG;

//    interval.SetSource(this);
    this.interval = interval;

    this.source_itemsel = new SELECT(item_sel_id, sourceUpdateItem, this);
    this.source_masksel = new SELECT(mask_sel_id, sourceUpdateMask, this);
    this.source_grsel = new SELECT(group_sel_id, sourceUpdateGroup, this);
    this.source_dbsel = new SELECT(db_sel_id, sourceUpdateDatabase, this);
    this.source_srvsel = new SELECT(srv_sel_id, sourceUpdateServer, this);

    this.config = null;
    this.updater = null;
    
    if (typeof opts == "object") this.opts = opts;
    else this.opts = new Object();
}

SOURCE.prototype.Init = function(updater, opts) {
    if (typeof opts == "undefined") opts = new Object;
    opts.reload = true;
    opts.confirm = true;

    this.updater = updater;
    this.interval.SetUpdater(updater);
    updater.RegisterControl(this);
    
    this.Update(opts);
}


SOURCE.prototype.AttachConfig = function(config) {
    this.config = config;
    this.interval.SetConfig(config);

    config.Register(this);

//    this.Apply();
}


SOURCE.prototype.ApplyConfig = function(subcall) {
    if (this.config) {
    	this.config.SetSource(this.cfg.db_server, this.cfg.db_name, this.cfg.db_group, this.cfg.db_mask);
	this.interval.ApplyConfig(true);

	if (typeof subcall == "undefined") {
//	    alert('save');
	    this.config.Save();
	}
    }
}

SOURCE.prototype.ReadConfig = function(opts) {
    if (this.config) {
	if (typeof opts == "undefined") opts = new Object;
	    
	opts.db_server = this.config.cfg.db_server;
	opts.db_name = this.config.cfg.db_name;
	opts.db_group = this.config.cfg.db_group;
	
	if (this.config.cfg.db_mask != null) {
    	    var mask = this.config.cfg.db_mask;
	    if (mask.search(',')>=0) {
		opts.db_mask = mask;
		opts.db_item = 0;
	    } else if (isNaN(mask)) {
		opts.db_mask = mask;
		opts.db_item = 0;
	    } else {
		opts.db_mask = -1;
		opts.db_item = mask;
	    }
	}
	
	this.interval.ReadConfig(opts);
	this.Update(opts);
    }
}


SOURCE.prototype.Apply = function() {
    this.ApplyConfig();
    if (this.updater) this.updater.Update();
}

SOURCE.prototype.RegisterGraph = function(graph) {
    this.graphs.push(graph);
}

SOURCE.prototype.Update = function(opts) {
    if (opts) {
	if (opts.reload) {
	    this.source_srvsel.Update(adei.GetSelectService("servers"), opts, opts?opts.db_server:null);
	} else if ((opts.reset)||(opts.db_server)) {
	    if (!opts.db_server) opts.db_server = this.source_srvsel.GetValue(0);

	    this.UpdateServer(opts.db_server, opts);
	    
	    if (this.source_srvsel.GetValue() != opts.db_server)
    		this.source_srvsel.SetValue(opts.db_server);
	} else if (opts.confirm) {
	    this.config.Confirm(this, 0);
	    this.config.Confirm(this, 1);
	    this.config.Confirm(this, 2);
	}
    }
}

SOURCE.prototype.UpdateServer = function(srv, opts) {
    if (typeof srv == "undefined") {
	srv = this.source_srvsel.GetValue();
    }

    with (this.cfg) {
	if (((opts)&&(opts.reload))||(srv != db_server)) {
	    db_server = srv;
	    
	    if (typeof opts == "undefined") opts = new Object;
	    opts.reload = true;
	    
	    this.source_dbsel.Update(adei.GetSelectService("databases", "db_server=" + db_server), opts, opts?opts.db_name:null);
	} else if ((opts)&&((opts.db_name)||(opts.reset))) {
	    if (!opts.db_name) opts.db_name = this.source_dbsel.GetValue(0);

	    this.UpdateDatabase(opts.db_name, opts);
	    
	    if (this.source_dbsel.GetValue() != opts.db_name)
    		this.source_dbsel.SetValue(opts.db_name);
	} else if ((opts)&&(opts.confirm)) {
	    this.config.Confirm(this, 0);
	    this.config.Confirm(this, 1);
	    this.config.Confirm(this, 2);
	}
    }
}


SOURCE.prototype.UpdateDatabase = function(db, opts) {
    with (this.cfg) {
	if (((opts)&&(opts.reload))||(db != db_name)) {
	    db_name = db;

	    if (typeof opts == "undefined") opts = new Object;
	    opts.reload = true;

	    this.source_grsel.Update(adei.GetSelectService("groups", "db_server=" + db_server + "&db_name=" + db_name), opts, opts?opts.db_group:null);
	} else if ((opts)&&((opts.db_group)||(opts.reset))) {
	    if (!opts.db_group) opts.db_group = this.source_grsel.GetValue(0);

	    this.UpdateGroup(opts.db_group, opts);

	    if (this.source_grsel.GetValue() != opts.db_group)
		this.source_grsel.SetValue(opts.db_group);
	} else if ((opts)&&(opts.confirm)) {
	    this.config.Confirm(this, 0);
	    this.config.Confirm(this, 1);
	    this.config.Confirm(this, 2);
	}
    }
}

/* Handling of Interval/Window updates. 
    a) INTERVAL value passed - UPDATING
    b) 'keep_window' in global or local options - KEEPING
    c) Group have changed - UPDATING
    d) 'reset' in local options - UPDATING 
    e) IGNORING


*/
SOURCE.prototype.UpdateGroup = function(group, opts) {
    if (typeof opts == "undefined") opts = new Object;

    if (!opts.experiment) {
	if ((opts.keep_window)||(this.opts.keep_window)) {
	    opts.experiment = this.config.cfg.experiment;
	    opts.window_width = this.config.win_width;
	    opts.window = "auto";
	}
    }
    
    with (this.cfg) {
	if (((opts)&&(opts.reload))||(group != db_group)) {
	    db_group = group;
	    
	    opts.reload = true;

	    this.source_itemsel.Update(adei.GetSelectService("items", "db_server=" + db_server + "&db_name=" + db_name + "&db_group=" + db_group), opts, opts?opts.db_item:null);
	    this.source_masksel.Update(adei.GetSelectService("masks", "db_server=" + db_server + "&db_name=" + db_name + "&db_group=" + db_group), opts, opts?opts.db_mask:null);

	    this.interval.Update(this.GetSourceProps(), opts);
	} else if ((opts)&&((opts.db_mask)||(opts.db_item)||(opts.experiment)||(opts.reset))) {
	    if ((opts.db_mask)||(opts.reset)) {
	        if (!opts.db_mask) opts.db_mask = this.source_masksel.GetValue(0);

		this.UpdateMask(opts.db_mask, opts);
		
		if (this.source_masksel.GetValue() != opts.db_mask)
		    this.source_masksel.SetValue(opts.db_mask);
	    } else if (opts.confirm) {
		this.config.Confirm(this, 0);
	    }
	    
	    if ((opts.db_item)||(opts.reset)) {
	        if (!opts.db_item) opts.db_item = this.source_itemsel.GetValue(0);

		this.UpdateItem(opts.db_item, opts);
	        
		if (this.source_itemsel.GetValue() != opts.db_item)
		    this.source_itemsel.SetValue(opts.db_item);
	    } else if (opts.confirm) {
		this.config.Confirm(this, 1);
	    }
	    
	    if ((opts.experiment)||(opts.reset)) {
		this.interval.Update(this.GetSourceProps(), opts);
	    } else if (opts.confirm) {
		this.config.Confirm(this, 2);
	    }
	} else if (opts.confirm) {
	    this.config.Confirm(this, 0);
	    this.config.Confirm(this, 1);
	    this.config.Confirm(this, 2);
	}
    }
}


SOURCE.prototype.UpdateMask = function(mask, opts) {
    with (this.cfg) {
	if (((opts)&&(opts.reload))||(mask != db_mask)) {
	    if (mask == -1) {
		db_mask = db_item;

		this.source_itemsel.Show();
		if (!this.module_on_display) this.fix_hidden = true;

		if (typeof this.geometry_cb == "function") {
		    this.geometry_cb(this.geometry_cbattr);
		}
	    } else {
		if (!isNaN(db_mask)) {
		    if (!this.module_on_display) this.fix_hidden = true;
		    this.source_itemsel.Hide();
		}
		
		db_mask = mask;

		if ((typeof opts == "undefined")||((opts)&&(opts.apply))) this.Apply();
	    }
	}
	
	if ((opts)&&(opts.confirm)) {
	    this.config.Confirm(this, 0);
	}
    }
}


SOURCE.prototype.UpdateItem = function(item, opts) {
    with (this.cfg) {
	if (((opts)&&(opts.reload))||(item != db_item)) {
	    db_item = item;
    	    db_mask = item;

	    if ((typeof opts == "undefined")||((opts)&&(opts.apply))) this.Apply();
	}

	if ((opts)&&(opts.confirm)) this.config.Confirm(this, 1);
    }
}


SOURCE.prototype.GetSourceProps = function() {
    return "db_server=" + this.cfg.db_server + "&db_name=" + this.cfg.db_name + "&db_group=" + this.cfg.db_group;
}

SOURCE.prototype.RegisterGeometryCallback = function(cb, cbattr) {
    this.interval.RegisterGeometryCallback(cb, cbattr);
    this.geometry_cb = cb;
    this.geometry_cbattr = cbattr;
}

SOURCE.prototype.FixHidden = function() {
		    // Fix IE bug, preventing unhiden hidden subnodes while within hidden element
    if ((this.fix_hidden)&&(this.module_on_display)) {
	if (isNaN(this.cfg.db_mask)) {
	    this.source_itemsel.Show();
	    this.source_itemsel.Hide();
	} else {
	    this.source_itemsel.Hide();
	    this.source_itemsel.Show();
	}
	this.fix_hidden = false;
    }
    this.interval.FixHidden();
}


SOURCE.prototype.onConfirmation = function(self) {
    return function () {
/*	if (this.alter_queue) {
	    this.Alter(this.alter_queue.type, this.alter_queue.direction);
	    this.alter_queue = false;
	}*/
	
	self.Apply();
    }
}

SOURCE.prototype.Alter = function (type, direction) {
    var cur = this.config.RequestConfirmation(this.onConfirmation(this));
    if (cur) {
/*
	if (cur == 1) {
	    this.alter_queue = new Object({
		type: type,
		direction: direction
	    });
	}
*/
	    // we are busy or something is completely wrong
	return adeiBusy("SOURCE update is failed", "source", this);
    }


    var opts = new Object({
	keep_window: true,
	confirm: true,
	reset: true
    });
    
    switch (type) {
	case 's':
	    if (direction > 0) this.source_srvsel.Next(true);
	    else this.source_srvsel.Prev(true);
	    this.UpdateServer(this.source_srvsel.GetValue(), opts);
	break;
	case 'd':
	    if (direction > 0) this.source_dbsel.Next(true);
	    else this.source_dbsel.Prev(true);
	    this.UpdateDatabase(this.source_dbsel.GetValue(), opts);
	break;
	case 'g':
	    if (direction > 0) this.source_grsel.Next(true);
	    else this.source_grsel.Prev(true);
	    this.UpdateGroup(this.source_grsel.GetValue(), opts);
	break;
	case 'm':
	    var val = this.source_masksel.GetValue();
	    if (parseInt(val) < 0) this.source_masksel.SetIndex(0);
	    else {
	        if (direction > 0) 
		    this.source_masksel.Next(true, function (idx,val) {
			if (parseInt(val) < 0) return true;
			return false;
		    });
		else 
		    this.source_masksel.Prev(true, function (idx,val) {
			if (parseInt(val) < 0) return true;
			return false;
		    });
	    }
	    
		// confirming item, experiment
	    this.config.Confirm(this, 1);
	    this.config.Confirm(this, 2);
	    
	    this.UpdateMask(this.source_masksel.GetValue(), opts);
	break;
	case 'i':

	    var val = this.source_masksel.GetValue();
	    if (parseInt(val) == -1) {
		if (direction > 0) this.source_itemsel.Next(true);
		else this.source_itemsel.Prev(true);

		this.config.Confirm(this, 0);
	    } else {
		this.source_itemsel.SetIndex(0);
		
		this.source_masksel.SetValue(-1);
	        this.UpdateMask(this.source_masksel.GetValue(), opts);
	    }
	    
	    this.config.Confirm(this, 2);
	    
	    
	    this.UpdateItem(this.source_itemsel.GetValue(), opts);
	break;
    }
}

SOURCE.prototype.NextServer = function() {
    this.Alter('s', 1);
}

SOURCE.prototype.PrevServer = function() {
    this.Alter('s', -1);
}

SOURCE.prototype.NextDatabase = function() {
    this.Alter('d', 1);
}

SOURCE.prototype.PrevDatabase = function() {
    this.Alter('d', -1);
}

SOURCE.prototype.NextGroup = function() {
    this.Alter('g', 1);
}

SOURCE.prototype.PrevGroup = function() {
    this.Alter('g', -1);
}

SOURCE.prototype.NextMask = function() {
    this.Alter('m', 1);
}

SOURCE.prototype.PrevMask = function() {
    this.Alter('m', -1);
}

SOURCE.prototype.NextItem = function() {
    this.Alter('i', 1);
}

SOURCE.prototype.PrevItem = function() {
    this.Alter('i', -1);
}



function sourceUpdateServer(src, srv, opts) {
    return src.UpdateServer(srv, opts);
}

function sourceUpdateDatabase(src, db, opts) {
    return src.UpdateDatabase(db, opts);
}

function sourceUpdateGroup(src, group, opts) {
    return src.UpdateGroup(group, opts);
}

function sourceUpdateMask(src, mask, opts) {
    return src.UpdateMask(mask, opts);
}

function sourceUpdateItem(src, item, opts) {
    return src.UpdateItem(item, opts);
}

function sourceFixHidden(source) {
    source.FixHidden();
}
